
import pygame
from Item import Item
import random
import glob

FONT_SIZE = 16
SHOWED_CHARS = 20
BLINK_CYCLES = 10
INPUT_COLOR = (255,255,255,255)
WRONG_COLOR = (255,0,0,255)
READING_COLOR = (255,255,0,255)
STATS_COLOR = (255,255,255,255)
STAT_HISTORY_CAPACITY = 80 # /20FPS => x seconds of history
TEXT_RESERVE = 20

class ConsoleItem(Item):
    def __init__(self, console):
        Item.__init__(self)
        self.console = console
        self.pos = (10,10)
        self.font = pygame.font.Font("fonts/FreeMonoBold.ttf", FONT_SIZE)
        self.dotWidth = self.font.size(".")[0]
        self.blink = 0
        self.reading = ""
        self.speed = 0
        self.statHistory = [(0,pygame.time.get_ticks())]
        self.addReading()

    def addReading(self):
        files = glob.glob("texts/*.txt")
        filename = random.choice(files)
        text = file(filename).read()
        self.reading += text.replace("\n", " ")

    def nextAnim(self):
        self.blink = (self.blink + 1) % (2 * BLINK_CYCLES)
        if len(self.console.line) + TEXT_RESERVE > len(self.reading):
            self.addReading()
        self.calcSpeed()

    def calcSpeed(self):
        """ Calc typing speed per minute.
        """
        history = self.statHistory[0]
        stat = (len(self.console.line), pygame.time.get_ticks())
        chars = self.countChars(history[0], stat[0])
        timeDiff = max(1, stat[1] - history[1])

        self.speed = 60000 * chars // timeDiff
        self.statHistory.append(stat)
        if len(self.statHistory) > STAT_HISTORY_CAPACITY:
            self.statHistory.pop(0)

    def countChars(self, start, end):
        """ Counts correctly typed chars between start and end position.
        """
        if end <= start:
            return 0
        ideal = self.reading[start:end]
        typed = self.console.line[start:end]
        chars = 0
        for c1, c2 in zip(ideal, typed):
            if c1 == c2:
                chars += 1
        return chars

    def getShowedReading(self, length):
        leftLimit = max(0, len(self.console.line) - SHOWED_CHARS)
        rightLimit = leftLimit + length
        return self.reading[leftLimit:rightLimit]

    def paintOn(self, screen):
        self.paintMistakes(screen)
        self.paintInput(screen)
        self.paintReading(screen)
        self.paintStats(screen)

    def paintMistakes(self, screen):
        ideal = self.getShowedReading(SHOWED_CHARS)
        text = self.console.line[-SHOWED_CHARS:]

        rect = (list(self.pos),
                (self.dotWidth, self.font.get_linesize() * 2))
        for i in range(len(text)):
            if text[i] != ideal[i]:
                rect[0][0] = self.pos[0] + i * self.dotWidth
                pygame.draw.rect(screen, WRONG_COLOR, rect)

    def paintInput(self, screen):
        text = self.console.line[-SHOWED_CHARS:]
        if self.blink < BLINK_CYCLES:
            text.append("_")
        text = "".join(text)

        surface = self.font.render(text, True, INPUT_COLOR)
        screen.blit(surface, self.pos)

    def paintReading(self, screen):
        length = screen.get_width() // self.dotWidth
        text = self.getShowedReading(length)
        surface = self.font.render(text, True, READING_COLOR)

        x = self.pos[0]
        y = self.pos[1] + self.font.get_linesize()
        screen.blit(surface, (x,y))

    def paintStats(self, screen):
        text = "speed: %3d length: %d" % (self.speed, len(self.console.line))
        surface = self.font.render(text, True, STATS_COLOR)

        x = self.pos[0]
        y = screen.get_height() - self.font.get_linesize()
        screen.blit(surface, (x,y))



