
import random
import pygame
from Item import Item
import Enemy

ENEMY_COLOR = (100,0,0,255)
ENEMY_RADIUS = 5
VERTICAL_BORDER = 80
DAMAGE = 4
MOVEMENT_SPEED = 2

ADDING_START_PERIOD = 30
ADDING_PERIOD_COEF = 0.996

class Enemies(Item):
    def __init__(self, target):
        Item.__init__(self)
        self.target = target
        self.enemies = []
        self.addingTime = 0
        self.addingPeriod = ADDING_START_PERIOD
        areaSize = pygame.display.get_surface().get_size()
        self.xBorders = (0, areaSize[0] - 1)
        self.yBorders = (VERTICAL_BORDER, areaSize[1] - 1 - VERTICAL_BORDER)

    def nextAnim(self):
        self.planAdding()
        self.moveEnemies()
        self.applyDamage()

    def planAdding(self):
        self.addingTime += 1
        if self.addingTime >= self.addingPeriod:
            self.addingTime = 0
            self.addEnemy()
            self.addingPeriod *= ADDING_PERIOD_COEF

    def moveEnemies(self):
        for enemy in self.enemies:
            dx = self.target.pos[0] - enemy.pos[0]
            dy = self.target.pos[1] - enemy.pos[1]
            newX, newY = enemy.pos
            if abs(dx) > 0:
                if random.randint(0,1) == 0:
                    newX += MOVEMENT_SPEED * dx // abs(dx)
            if abs(dy) > 0:
                if random.randint(0,4) == 0:
                    newY += MOVEMENT_SPEED * dy // abs(dy)
            else:
                newY += MOVEMENT_SPEED * random.randint(-1,1)

            enemy.pos = newX, newY

    def applyDamage(self):
        shieldRadius = self.target.getShieldRadius()
        sqShield = shieldRadius * shieldRadius
        i = 0
        while i < len(self.enemies):
            enemy = self.enemies[i]
            if self.isShielded(sqShield, enemy):
                self.target.damage += DAMAGE
                del self.enemies[i]
            else:
                i += 1

    def isShielded(self, sqShield, enemy):
        dx = self.target.pos[0] - enemy.pos[0]
        dy = self.target.pos[1] - enemy.pos[1]
        sqDistance = dx * dx + dy * dy
        return sqDistance <= sqShield

    def addEnemy(self):
        x = random.choice(self.xBorders)
        y = random.randint(self.yBorders[0], self.yBorders[1])

        enemy = Enemy.Enemy((x,y))
        self.enemies.append(enemy)

    def paintOn(self, screen):
        for enemy in self.enemies:
            pos = enemy.pos
            pygame.draw.circle(screen, ENEMY_COLOR, pos, ENEMY_RADIUS)


