
import pygame
import Console
import ConsoleItem
import Magican
import Enemies

FONT_SIZE = 30
WELCOME_SHOW = 100
DEAD_SHOW = 50
TEXT_COLOR = (255,255,255,255)

class Game:
    def __init__(self):
        self.done = False
        self.items = []
        self.font = pygame.font.Font(None, FONT_SIZE)

        self.console = Console.Console()
        consoleItem = ConsoleItem.ConsoleItem(self.console)

        self.magican = Magican.Magican(consoleItem)

        enemies = Enemies.Enemies(self.magican)

        self.items.append(consoleItem)
        self.items.append(self.magican)
        self.items.append(enemies)
        self.welcomeCount = 0
        self.deadCount = 0

    def isDone(self):
        return self.done

    def loop(self):
        self.handleEvents()

        if self.magican.isDead():
            self.deadScreen()
        else:
            self.driveAnim()
            self.paint()

    def deadScreen(self):
        self.showText("They got you.")
        pygame.display.flip()

        self.deadCount += 1
        if self.deadCount >= DEAD_SHOW:
            self.__init__()

    def showText(self, text):
        screen = pygame.display.get_surface()
        surface = self.font.render(text, True, TEXT_COLOR)
        y = screen.get_height() // 3
        x = (screen.get_width() - surface.get_width()) // 2
        screen.blit(surface, (x,y))

    def handleEvents(self):
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                self.done = True
            elif event.type == pygame.KEYDOWN:
                if event.key == pygame.K_ESCAPE:
                    self.done = True
                elif event.key == pygame.K_RETURN:
                    if self.magican.isDead():
                        self.deadCount = DEAD_SHOW
                else:
                    self.console.handleKey(event)

    def driveAnim(self):
        for item in self.items:
            item.nextAnim()

    def paint(self):
        screen = pygame.display.get_surface()
        screen.fill((180,130,0,255))

        if self.welcomeCount < WELCOME_SHOW:
            self.welcomeCount += 1
            self.showText("Protect yourself by correct spell typing.")

        for item in self.items:
            item.paintOn(screen)
        pygame.display.flip()

