
import pygame
from Item import Item
import Anim

VULNERABLE_ZONE = 20
SHIELD_COLOR = (0, 200, 100, 255)
WEAK_COLOR = (200, 50, 50, 255)
REGENERATION_RATE = 1
ANIM_PERIOD = 400

class Magican(Item):
    def __init__(self, speedometer):
        Item.__init__(self)
        self.speedometer = speedometer
        self.power = 0
        self.damage = 0
        areaSize = pygame.display.get_surface().get_size()
        self.pos = (areaSize[0] // 2, areaSize[1] // 2)
        self.anim = Anim.Anim("animation/mag.anim")

        animSize = self.anim.getSize()
        self.anim.pos = (self.pos[0] - animSize[0] // 2,
                self.pos[1] - animSize[1] // 2)
        self.animTime = 0

    def getShieldRadius(self):
        return max(VULNERABLE_ZONE, self.power - self.damage)

    def getShieldPower(self):
        return max(0, self.power - self.damage - VULNERABLE_ZONE)

    def isDead(self):
        return self.damage > 0 and self.getShieldPower() == 0

    def nextAnim(self):
        self.power = max(1, self.speedometer.speed // 3)
        self.damage -= REGENERATION_RATE
        if self.damage < 0:
            self.damage = 0

        self.animTime += self.speedometer.speed
        while self.animTime >= ANIM_PERIOD:
            self.animTime -= ANIM_PERIOD
            self.anim.nextAnim()

    def paintOn(self, screen):
        shieldWidth = self.getShieldPower()

        color = SHIELD_COLOR
        if shieldWidth == 0:
            color = WEAK_COLOR
        pygame.draw.circle(screen, color, self.pos, self.power, shieldWidth)
        if self.damage > 0:
            pygame.draw.circle(screen, WEAK_COLOR, self.pos,
                    self.power, min(self.power, self.damage))

        self.paintFigure(screen)

    def paintFigure(self, screen):
        self.anim.paintOn(screen)



